function [CoreTensor, U] = HOSVD(T,ratioArray)
%   Description:    This function is used to compute the SVD of a high
%   order tensor, and also can be used to reduce the dimension by k, the 
%   threshold that indicate the feature proportion preserved.
%
%   Argument:          
%           T            -The high-order tensor
%           k            -The proportion of feature need to keep.
%
%   Returns:
%           U            -A cell stored the left sigular matrix after SVD.
%           CoreTensor   -The core tensor after SVD
%   see also svd, tensor2matrix

%   Author: Aimery Kong    
%   3/10/2010

I = size(T);
if nargin == 1
    ratioArray = ones(1,length(I));
else
    if length(I) ~= length(ratioArray)
        ratioArray = [ratioArray ones(1,length(I)-length(ratioArray))];
    end
end

%% Caculating Tensor SVD
Inew = [];
U = cell(length(I));
for i = 1:length(I);
    TEMP = tensor2matrix(T,i);
    TEMP = TEMP*TEMP';
    [U{i} S V] = svd(TEMP); % TEMP=U*S*V'
    
    stmp = sum(S,1);
    for j = 1:size(S,1)
        if sum(stmp(1:j))/sum(stmp) >= ratioArray(i)
            break;
        end
    end
    u{i} = j;
    Inew = [Inew u{i}];
    U{i} = U{i}(:,1:u{i});
end
clear S V;

%% Caculating the core tensor
%%{
CoreTensor = T;
Inew = size(T);
for i = 1:length(I)
    CoreTensor = tensor2matrix(CoreTensor,i);
    CoreTensor = U{i}'*CoreTensor;
    Inew(i) = size(U{i},2);
    CoreTensor = matrix2tensor(CoreTensor,Inew,i);
end
%%}
%{
TEMP = U{length(I)};
for i = (length(I)-1):-1:2
    TEMP = kron(TEMP,U{i});
end
TT = tensor2matrix(T,1);
a = 1;
TEMP = U{1}'*TT*TEMP;
CoreTensor = matrix2tensor(TEMP,Inew,1);
clear TEMP Inew
%}
