function Result = Hoprod(T,S,type,mode)
% Description:
% Hoprod is a function used to caculate the product between tensor
% and tensor, or tensor and matrix, or tensor and vector. It is useful and
% convinient to carry out the product stuff. However, in tensor-tensor
% product, it is only valid to do the product between 2-way tensior, i.e.
% matrix. Because high order outer product is time expensive and space
% expensive, and 2-way tensor outer product is enough to do common
% research.
%
% Usage  -----------------------------------------------------------------
% 
%
% Argument:
%       T       - Input a tensor
%       S       - Input the mulitiplier, maybe a tensor as well, or a
%                 matrix, or a vector.
%       type    - Input the type in 'ten' 'mat' or 'vec' stand for
%                 tensor-tensor, tensor-matrix and tensor-vector product
%                 respectively. It is OK to input 'tensor', 'matrix' and
%                 'vector'.
%       mode    - This is the mode that the caculation is going to happen
%                 on. It is a requirement in tensor-matrix and tensor-vector 
%                 type.
%
% Returns:
%       Result  - Return the new tensor after caculation.
%
% see also tensor2matrix, matrix2tensor, reshape, squeeze
%

% Author: Aimery Kong
% Date:   31/10/2010

%%
if nargin < 2
    disp('Error in the arguments number!');
    return;
end

%% tensor product or outer product--Only support the strict 2-way matrix outer product
if nargin == 2 || strcmp(type,'ten') || strcmp(type,'tensor')
    It = size(T);
    Is = size(S);
    Result = T;
    if Is(2) == 1 && length(size(T)) == 3
        for i = 1:Is(1)
            Result(:,:,:,i) = T*S(i);
        end
        return;
    end
    
    Result = zeros([It Is]);
    for i = 1:size(Result,1)
        for j = 1:size(Result,2)
            for n = 1:size(Result,3)
                for m = 1:size(Result,4)
                    Result(i,j,n,m) = T(i,j)*S(n,m);
                end
            end
        end        
    end
    Result = squeeze(Result);
    return;
end    

%% tensor-vector product on mode 'mode'
if strcmp(type,'vec') || strcmp(type,'vector')
    Result = tensor2matrix(T,mode); 
    I = size(T);
    if length(I) ~= 2
        Result = S'*Result;
        if mode == 1
            Inew = I(2:end);
        elseif mode == length(I)
            Inew = I(1:end-1);
        else
            Inew = [I(1:mode-1) I(mode+1:end)];
        end
        Result = reshape(Result,Inew);
    else
        Result = S'*Result;
    end
    Result = squeeze(Result);
    return;
end


%% tensor-matrix product on mode 'mode'
if strcmp(type,'mat') || strcmp(type,'matrix')
    Result = tensor2matrix(T,mode);
    I = size(T);
    sz = size(S);
    Inew = I;
    Inew(mode) = sz(1);
    Result = S*Result;
    Result = matrix2tensor(Result,Inew,mode);
end
return;